﻿using TypographyDataModels.Models;
using TypographyContracts.BindingModels;
using TypographyContracts.ViewModels;
using System.Xml.Linq;
using System.Runtime.Serialization;

namespace TypographyFileImplement.Models
{
    [DataContract]
    public class Printed : IPrintedModel
    {
        [DataMember]
        public int Id { get; private set; }
        [DataMember]
        public string PrintedName { get; private set; } = string.Empty;
        [DataMember]
        public double Price { get; private set; }

        public Dictionary<int, int> Components { get; private set; } = new();

        private Dictionary<int, (IComponentModel, int)>? _printedComponents = null;
        [DataMember]
        public Dictionary<int, (IComponentModel, int)> PrintedComponents
        {
            get
            {
                if (_printedComponents == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _printedComponents = Components.ToDictionary(x => x.Key, y =>
                    ((source.Components.FirstOrDefault(z => z.Id == y.Key) as IComponentModel)!, y.Value));
                }
                return _printedComponents;
            }
        }
        public static Printed? Create(PrintedBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Printed()
            {
                Id = model.Id,
                PrintedName = model.PrintedName,
                Price = model.Price,
                Components = model.PrintedComponents.ToDictionary(x => x.Key, x
               => x.Value.Item2)
            };
        }
        public static Printed? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Printed()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                PrintedName = element.Element("PrintedName")!.Value,
                Price = Convert.ToDouble(element.Element("Price")!.Value),
                Components = element.Element("PrintedComponents")!.Elements("PrintedComponent").ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value), x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        public void Update(PrintedBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            PrintedName = model.PrintedName;
            Price = model.Price;
            Components = model.PrintedComponents.ToDictionary(x => x.Key, x => x.Value.Item2);
            _printedComponents = null;
        }
        public PrintedViewModel GetViewModel => new()
        {
            Id = Id,
            PrintedName = PrintedName,
            Price = Price,
            PrintedComponents = PrintedComponents
        };
        public XElement GetXElement => new("Printed",
        new XAttribute("Id", Id),
        new XElement("PrintedName", PrintedName),
        new XElement("Price", Price.ToString()),
        new XElement("PrintedComponents", Components.Select(x => new XElement("PrintedComponent", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()));
    }
}
