﻿using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Runtime.Serialization;
using TypographyContracts.BindingModels;
using TypographyContracts.ViewModels;
using TypographyDatabaseImplements.Models;
using TypographyDataModels.Models;

namespace TypographyDatabaseImplement.Models
{
    [DataContract]
    public class Component : IComponentModel
    {
        [DataMember]
        public int Id { get; private set; }
        [Required]
        [DataMember]
        public string ComponentName { get; private set; } = string.Empty;
        [Required]
        [DataMember]
        public double Cost { get; set; }
        [ForeignKey("ComponentId")]
        public virtual List<PrintedComponent> PrintedComponents { get; set; } = new();
        public static Component? Create(ComponentBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Component()
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost
            };
        }
        public static Component Create(ComponentViewModel model)
        {
            return new Component
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost
            };
        }
        public void Update(ComponentBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ComponentName = model.ComponentName;
            Cost = model.Cost;
        }
        public ComponentViewModel GetViewModel => new()
        {
            Id = Id,
            ComponentName = ComponentName,
            Cost = Cost
        };
    }
}
