﻿using TypographyContracts.BindingModels;
using TypographyContracts.ViewModels;
using TypographyDataModels.Enums;
using System.ComponentModel.DataAnnotations;
using TypographyDatabaseImplements.Models;
using System.ComponentModel.DataAnnotations.Schema;
using TypographyDataModels.Models;
using System.Runtime.Serialization;

namespace TypographyDatabaseImplement.Models
{
    [DataContract]
    public class Order : IOrderModel
    {
        [DataMember]
        public int Id { get; set; }

        [Required]
        [DataMember]
        public int PrintedId { get; set; }
        [Required]
        [DataMember]
        public int ClientId { get; private set; }
        [DataMember]
        public int? ImplementerId { get; private set; }

        [Required]
        [DataMember]
        public int Count { get; set; }

        [Required]
        [DataMember]
        public double Sum { get; set; }

        [Required]
        [DataMember]
        public OrderStatus Status { get; set; }

        [Required]
        [DataMember]
        public DateTime DateCreate { get; set; }
        [DataMember]
        public DateTime? DateImplement { get; set; }

        public virtual Printed? Printed { get; set; }
        public virtual Client? Client { get; set; }
        public Implementer? Implementer { get; private set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                PrintedId = model.PrintedId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PrintedId = PrintedId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            PrintedName = Printed?.PrintedName ?? string.Empty,
            ClientFIO = Client?.ClientFIO ?? string.Empty,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
        };
    }
}